@@ -183,7 +183,7 @@ if ($output eq 'h') {
     #
 
     @hashinfo = gen_perfect_hash(\%tokens);
-    if (!defined(@hashinfo)) {
+    if (!@hashinfo) {
 	die "$0: no hash found\n";
     }
 
